class Distributor
  module Actions

    class Subscribe < Action
      FORBIDDEN_TEMPLATE = Actions.template("subscribe.forbidden")
      SUCCESS_TEMPLATE = Actions.template("subscribe.success")
      WAIT_USER_TEMPLATE = Actions.template("subscribe.wait_user")
      WAIT_MODO_TEMPLATE = Actions.template("subscribe.wait_modo")

      def handle(list:, to:, attributes:)
        register =
          begin
            Email.register!(mailinglist: list, name: to.from_name, email: to.from).save
          rescue => err
            $logger.error err.message
            nil
          end
        if register
          if !register.reader?
            handle_wait_validation(list: list, to: to, register: register)
          else
            handle_subscribed(list: list, to: to, register: register)
          end
        else
          handle_403(list: list, to: to)
        end
      end

      def handle_wait_validation(list:, to:, register:)
        $logger.debug "Subscribe#handle_wait_validation on #{list.email} for #{to.from}"
        body = WAIT_USER_TEMPLATE.result binding
        @distributor.distribute(to.to_response(list: list, to: to, body: body))

        modo = list.enabled_modos.first
        body = WAIT_MODO_TEMPLATE.result binding
        @distributor.distribute(Protocols::Mail.build(subject: "ML #{list.name} requires validaton for #{to.from}", list: list, to: modo.email, body: body))
      end

      def handle_subscribed(list:, to:, register:)
        $logger.debug "Subscribe#handle_subscribed on #{list.email} for #{to.from}"
        $logger.debug register.inspect
        body = SUCCESS_TEMPLATE.result binding
        @distributor.distribute(to.to_response(list: list, to: to, body: body))
      end

      def handle_403(list:, to:)
        $logger.debug "Subscribe#handle_403 on #{list.email} for #{to.from}"
        body = FORBIDDEN_TEMPLATE.result binding
        @distributor.distribute(to.to_response(list: list, to: to, body: body))
      end
    end

    class Unsubscribe < Action
      SUCCESS_TEMPLATE = Actions.template("unsubscribe.success")

      def handle(list:, to:, attributes:)
        Email.unregister!(mailinglist: list, email: to.from)
        body = SUCCESS_TEMPLATE.result binding
        @distributor.distribute(to.to_response(list: list, to: to, body: body))
      end
    end

    class Help < Action
      HELP_TEMPLATE = Actions.template("help")
      def handle(list:, to:, attributes:)
        body = HELP_TEMPLATE.result binding
        @distributor.distribute(to.to_response(list: list, to: to, body: body))
      end
    end

    # This distribute the mail among the readers
    class Distribute < Action
      def handle(list:, to:, attributes:)
        if !list
          warn "invalid email writer for #{mail.from} on #{mail.to}"
          return nil
        end

        list.enabled_readers.each do |reader|
          to_distrib = to.to_redistribute(list: list, dest: reader)
          @distributor.distribute(to_distrib)
        end
      end
    end

  end
end
