require "net/imap"

class Protocols::Imap
  def initialize
    @imap = Net::IMAP.new(ENV["IMAP_HOST"], ENV["IMAP_PORT"], ssl: true)
    @imap.authenticate('PLAIN', ENV["IMAP_USER"], ENV["IMAP_PASSWORD"])
    @imap.select("INBOX")
  end

  def fetch
    @imap.check
    id = @imap.search(%w[NOT SEEN]).last
    return nil if id.nil?

    imap_mail = @imap.fetch(
      id, [Protocols::ENVELOPE, Protocols::UID, Protocols::HEADERS, Protocols::BODYTEXT, Protocols::MSG_ID],
    ).first
    return nil if imap_mail.nil?

    mail = Protocols::Mail.new(imap_mail: imap_mail)
    puts "READ #{mail.from}\t -> #{mail.to}:\t#{mail.subject}"

    mail
  end

  def clean
    puts "MARK ALL NOT SEEN as SEEN"
    @imap.search(["NOT", "SEEN"]).each { @imap.store _1, "+FLAGS", [Net::IMAP::SEEN] }
  end

  def seen!(imap_mail)
    puts "MARK #{imap_mail.attr['UID']} as SEEN"
    @imap.uid_store imap_mail.attr["UID"], "+FLAGS", [Net::IMAP::SEEN]
  end
end
